@@ -1,9 +1,11 @@
 package main
 
 import (
+	"fmt"
 	"math"
 
 	"github.com/projectdiscovery/katana/pkg/engine/standard"
+	"github.com/projectdiscovery/katana/pkg/output"
 	"github.com/projectdiscovery/katana/pkg/types"
 	"github.com/projectdiscovery/katana/pkg/utils/queue"
 )
@@ -17,6 +19,8 @@ type goIntegrationTest struct{}
 // Execute executes a test case and returns an error if occurred
 // Execute the docs at ../README.md if the code stops working for integration.
 func (h *goIntegrationTest) Execute() error {
+	var crawledURLs []string
+
 	options := &types.Options{
 		MaxDepth:     1,
 		FieldScope:   "rdn",
@@ -24,6 +28,9 @@ func (h *goIntegrationTest) Execute() error {
 		RateLimit:    150,
 		Verbose:      debug,
 		Strategy:     queue.DepthFirst.String(),
+		OnResult: func(r output.Result) {
+			crawledURLs = append(crawledURLs, r.Request.URL)
+		},
 	}
 	crawlerOptions, err := types.NewCrawlerOptions(options)
 	if err != nil {
@@ -36,5 +43,12 @@ func (h *goIntegrationTest) Execute() error {
 	}
 	defer crawler.Close()
 	var input = "https://public-firing-range.appspot.com"
-	return crawler.Crawl(input)
+	err = crawler.Crawl(input)
+	if err != nil {
+		return err
+	}
+	if len(crawledURLs) == 0 {
+		return fmt.Errorf("no URLs crawled")
+	}
+	return nil
 }
