@@ -1,14 +1,16 @@
 package runner
 
 import (
-	"github.com/pkg/errors"
 	"github.com/projectdiscovery/gologger"
 	"github.com/projectdiscovery/katana/pkg/engine"
 	"github.com/projectdiscovery/katana/pkg/engine/hybrid"
+	"github.com/projectdiscovery/katana/pkg/engine/parser"
 	"github.com/projectdiscovery/katana/pkg/engine/standard"
 	"github.com/projectdiscovery/katana/pkg/types"
+	errorutil "github.com/projectdiscovery/utils/errors"
 	fileutil "github.com/projectdiscovery/utils/file"
 	"go.uber.org/multierr"
+	updateutils "github.com/projectdiscovery/utils/update"
 )
 
 // Runner creates the required resources for crawling
@@ -30,11 +32,22 @@ func New(options *types.Options) (*Runner, error) {
 		return nil, nil
 	}
 
+	if !options.DisableUpdateCheck {
+		latestVersion, err := updateutils.GetVersionCheckCallback("katana")()
+		if err != nil {
+			if options.Verbose {
+				gologger.Error().Msgf("katana version check failed: %v", err.Error())
+			}
+		} else {
+			gologger.Info().Msgf("Current katana version %v %v", version, updateutils.GetVersionDescription(version, latestVersion))
+		}
+	}
+
 	if err := initExampleFormFillConfig(); err != nil {
-		return nil, errors.Wrap(err, "could not init default config")
+		return nil, errorutil.NewWithErr(err).Msgf("could not init default config")
 	}
 	if err := validateOptions(options); err != nil {
-		return nil, errors.Wrap(err, "could not validate options")
+		return nil, errorutil.NewWithErr(err).Msgf("could not validate options")
 	}
 	if options.FormConfig != "" {
 		if err := readCustomFormConfig(options); err != nil {
@@ -43,12 +56,12 @@ func New(options *types.Options) (*Runner, error) {
 	}
 	crawlerOptions, err := types.NewCrawlerOptions(options)
 	if err != nil {
-		return nil, errors.Wrap(err, "could not create crawler options")
+		return nil, errorutil.NewWithErr(err).Msgf("could not create crawler options")
 	}
 
-	var (
-		crawler engine.Engine
-	)
+	parser.InitWithOptions(options)
+
+	var crawler engine.Engine
 
 	switch {
 	case options.Headless:
@@ -57,7 +70,7 @@ func New(options *types.Options) (*Runner, error) {
 		crawler, err = standard.New(crawlerOptions)
 	}
 	if err != nil {
-		return nil, errors.Wrap(err, "could not create standard crawler")
+		return nil, errorutil.NewWithErr(err).Msgf("could not create standard crawler")
 	}
 	runner := &Runner{options: options, stdin: fileutil.HasStdin(), crawlerOptions: crawlerOptions, crawler: crawler}
 
